from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel
import logging
import requests
from requests.compat import urljoin

logger = logging.getLogger(__name__)


class AWX(NetBoxModel):
    name = models.CharField(max_length=100, help_text=("AWX instance name"))
    url = models.CharField(max_length=200, verbose_name=("URL"))
    token = models.CharField(max_length=200, verbose_name=("Token"))

    class Meta:
        ordering = ("name", "url", "token")

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:netbox_awx_plugin:awx", args=[self.pk])

    def get_headers(self):
        return {
            "Authorization": "Bearer {}".format(self.token),
            "Content-type": "application/json",
        }


class AWXInventory(NetBoxModel):
    awx = models.ForeignKey(
        to="netbox_awx_plugin.AWX", on_delete=models.CASCADE, related_name="inventories", help_text=("AWX instance")
    )
    inventory_id = models.IntegerField()
    enabled = models.BooleanField(default=True)

    class Meta:
        ordering = ("awx", "inventory_id", "enabled")

    def get_absolute_url(self):
        return reverse("plugins:netbox_awx_plugin:awxinventory", args=[self.pk])

    def get_host(self, hostname):
        params = {"name": hostname}
        r = requests.get(
            url=urljoin(self.awx.url, "/api/v2/inventories/{}/hosts/".format(self.inventory_id)),
            params=params,
            headers=self.awx.get_headers(),
            verify=False,
        )
        r.raise_for_status()
        if r.json()["count"] == 1:
            host = r.json()["results"][0]
            return host
        else:
            return None

    def create_host(self, data):
        logger.info("Creating host with: {}".format(data))
        r = requests.post(
            url=urljoin(self.awx.url, "/api/v2/inventories/{}/hosts/".format(self.inventory_id)),
            headers=self.awx.get_headers(),
            json=data,
            verify=False,
        )
        r.raise_for_status()

    def update_host(self, host_id, data):
        logger.info("Updating host with: {}".format(data))
        r = requests.put(
            url=urljoin(self.awx.url, "/api/v2/hosts/{}/".format(host_id)),
            headers=self.awx.get_headers(),
            json=data,
            verify=False,
        )
        r.raise_for_status()

    def delete_host(self, hostname):
        host = self.get_host(hostname)
        r = requests.delete(
            url=urljoin(self.awx.url, "/api/v2/hosts/{}/".format(host["id"])),
            headers=self.awx.get_headers(),
            verify=False,
        )
        r.raise_for_status()

    def get_group(self, name):
        params = {"name": name}
        r = requests.get(
            url=urljoin(self.awx.url, "/api/v2/groups/"),
            params=params,
            headers=self.awx.get_headers(),
            verify=False,
        )
        r.raise_for_status()
        return r.json()["results"][0] if len(r.json()["results"]) > 0 else None

    def update_group(self, name, data):
        logger.info("Updating group {} with: {}".format(name, data))
        group = self.get_group(name)
        r = requests.put(
            url=urljoin(self.awx.url, "/api/v2/groups/{}/".format(group["id"])),
            headers=self.awx.get_headers(),
            json=data,
            verify=False,
        )
        r.raise_for_status()

    def create_group(self, data):
        logger.info("Creating group with {}".format(data))
        r = requests.post(
            url=urljoin(self.awx.url, "/api/v2/inventories/{}/groups/".format(self.inventory_id)),
            headers=self.awx.get_headers(),
            json=data,
            verify=False,
        )
        r.raise_for_status()

    def delete_group(self, name):
        logger.info("Deleting group {}".format(name))
        group = self.get_group(name)
        if not group is None:
            r = requests.delete(
                url=urljoin(self.awx.url, "/api/v2/groups/{}/".format(group["id"])),
                headers=self.awx.get_headers(),
                verify=False,
            )
            r.raise_for_status()

    def associate_host_group(self, host_id, group_id):
        logger.info("Associating host {} to group {}".format(host_id, group_id))
        data = {"id": group_id}
        r = requests.post(
            url=urljoin(self.awx.url, "/api/v2/hosts/{}/groups/".format(host_id)),
            headers=self.awx.get_headers(),
            json=data,
            verify=False,
        )
        r.raise_for_status()

    def disassociate_host_group(self, host_id, group_id):
        logger.info("Disassociating host {} from group {}".format(host_id, group_id))
        r = requests.post(
            url=urljoin(self.awx.url, "/api/v2/hosts/{}/groups/".format(host_id)),
            headers=self.awx.get_headers(),
            json={"id": group_id, "disassociate": "True"},
            verify=False,
        )
        r.raise_for_status()

    def get_host_groups(self, hostname):
        r = requests.get(
            url=urljoin(self.awx.url, "/api/v2/hosts/{}/groups/".format(hostname)),
            headers=self.awx.get_headers(),
            verify=False,
        )
        r.raise_for_status()
        return r.json()["results"]


class AWXHost(models.Model):

    name = models.CharField(max_length=100, help_text=("Host name"))
    description = models.CharField(max_length=256, help_text=("Description"))
    enabled = models.BooleanField(default=True)
    variables = models.JSONField()

    class Meta:
        managed = False

    def save(self, *args, **kwargs):
        pass
