import logging
from datetime import timedelta
from dcim.models import Device, DeviceRole, DeviceType, Site
from ipam.models import Prefix
from virtualization.models import VirtualMachine
from extras.models import Tag
from core.models import Job
from .models import AWXInventory
from .serializers import (
    serializers_dict,
    AWXHostSerializer,
    AWXGroupSerializer,
)
from django_rq import get_queue
from rq import get_current_job, Queue
from requests.exceptions import RequestException, HTTPError, ConnectionError, Timeout
from urllib3.exceptions import InsecureRequestWarning
import urllib3

# Set up logging
logger = logging.getLogger(__name__)

# Suppress only the single InsecureRequestWarning from urllib3
urllib3.disable_warnings(InsecureRequestWarning)

# Configuration for retry mechanism
MAX_RETRIES = 5
INITIAL_DELAY = 1
BACKOFF_FACTOR = 2

def reschedule_job(job, delay_seconds, attempt):
    """
    Reschedules the current job with the same job ID and the specified delay.
    """
    try:
        job.meta['attempt'] = attempt
        job.save_meta()
        
        # Remove the job from its current queue and re-add it with a delay
        queue = Queue(job.origin, connection=job.connection)
        queue.enqueue_in(timedelta(seconds=delay_seconds), job.func, *job.args, **job.kwargs, job_id=job.id, meta=job.meta)
        
        logger.info(f"Job {job.id} rescheduled with delay of {delay_seconds} seconds for attempt {attempt}.")
    except Exception as e:
        logger.error(f"Failed to reschedule job {job.id}: {e}")
        raise

def retry_on_failure(task_name):
    """
    A decorator that retries a function if specific network-related exceptions occur.
    It retries up to MAX_RETRIES times, and if all attempts fail, it raises an exception
    so that Django RQ can mark the job as failed.
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            job = get_current_job()
            attempt = job.meta.get('attempt', 1)

            try:
                return func(*args, **kwargs)
            except (RequestException, HTTPError, ConnectionError, Timeout) as e:
                logger.error(f"Error occurred during {task_name} on attempt {attempt}: {e}")
                if attempt < MAX_RETRIES:
                    delay_seconds = INITIAL_DELAY * (BACKOFF_FACTOR ** (attempt - 1))
                    job.timeout = delay_seconds + 60  # Adjust the timeout for the next attempt
                    reschedule_job(job, delay_seconds, attempt + 1)
                else:
                    logger.error(f"Task {task_name} failed on the last attempt: {e}")
                    # Let the exception propagate so the job is marked as failed
                    raise
        return wrapper
    return decorator

group_prefixes = {
    Site: "site_",
    DeviceRole: "devicerole_",
    DeviceType: "devicetype_",
    Prefix: "prefix_",
    Tag: "tag_"
}

@retry_on_failure('sync_host')
def sync_host(inventory, sender, instance):
    """
    Synchronizes a host in AWX inventory.
    """
    serializer_class = serializers_dict[sender]
    serializer = serializer_class(instance)
    host = inventory.get_host(serializer.data["name"])
    if host is None:
        inventory.create_host(serializer.data)
        host = inventory.get_host(serializer.data["name"])
    else:
        host_serializer = AWXHostSerializer(data=host)
        if host_serializer.is_valid():
            if host_serializer.validated_data != serializer.data:
                inventory.update_host(host["id"], serializer.data)
        else:
            logger.error(f"Invalid data for host_serializer: {host_serializer.errors}")

    current_groups = host["summary_fields"]["groups"]["results"]

    if hasattr(instance, 'site') and instance.site:
        sync_host_group_association(inventory, host, Site, instance.site, current_groups)
    if hasattr(instance, 'role') and instance.role:
        sync_host_group_association(inventory, host, DeviceRole, instance.role, current_groups)
    if isinstance(instance, Device) and instance.device_type:
        sync_host_group_association(inventory, host, DeviceType, instance.device_type, current_groups)
    if hasattr(instance, 'tags'):
        tags = instance.tags.all()
        for tag in tags:
            sync_host_group_association(inventory, host, Tag, tag, current_groups)

    disassociate_removed_groups(inventory, host, instance, current_groups)

@retry_on_failure('sync_host_group_association')
def sync_host_group_association(inventory, host, object_type, instance, current_groups):
    """
    Synchronizes host group associations in AWX inventory.
    """
    serializer_class = serializers_dict[object_type]
    serializer = serializer_class(instance)
    group_name = serializer.data["name"]

    relevant_groups = [
        group for group in current_groups if group["name"].startswith(group_prefixes[object_type])
    ]

    if group_name not in [group["name"] for group in relevant_groups]:
        group = inventory.get_group(group_name)
        if group:
            inventory.associate_host_group(host["id"], group["id"])
            logger.info(f"Host {host['name']} associated with group {group_name}.")

@retry_on_failure('disassociate_removed_groups')
def disassociate_removed_groups(inventory, host, instance, current_groups):
    """
    Disassociates removed groups from a host in AWX inventory.
    """
    valid_group_names = set()

    if hasattr(instance, 'site') and instance.site:
        valid_group_names.add(f"{group_prefixes[Site]}{instance.site.slug.replace('-', '_')}")
    if hasattr(instance, 'role') and instance.role:
        valid_group_names.add(f"{group_prefixes[DeviceRole]}{instance.role.slug.replace('-', '_')}")
    if isinstance(instance, Device) and instance.device_type:
        valid_group_names.add(f"{group_prefixes[DeviceType]}{instance.device_type.slug.replace('-', '_')}")
    if hasattr(instance, 'tags'):
        tags = instance.tags.all()
        valid_group_names.update(
            f"{group_prefixes[Tag]}{tag.slug.replace('-', '_')}" for tag in tags
        )

    for group in current_groups:
        if group["name"] not in valid_group_names:
            inventory.disassociate_host_group(host["id"], group["id"])
            logger.info(f"Host {host['name']} disassociated from group {group['name']}.")

@retry_on_failure('delete_host')
def delete_host(inventory, sender, instance):
    """
    Deletes a host from AWX inventory.
    """
    serializer_class = serializers_dict[sender]
    serializer = serializer_class(instance)
    inventory.delete_host(serializer.data["name"])

@retry_on_failure('sync_group')
def sync_group(inventory, sender, instance):
    """
    Synchronizes a group in AWX inventory.
    """
    serializer_class = serializers_dict[sender]
    serializer = serializer_class(instance)
    group = inventory.get_group(serializer.data["name"])
    if group is None:
        inventory.create_group(serializer.data)
    else:
        group_serializer = AWXGroupSerializer(data=group)
        if group_serializer.is_valid():
            if group_serializer.validated_data != serializer.data:
                inventory.update_group(serializer.data["name"], serializer.data)
        else:
            logger.error(f"Invalid data for group_serializer: {group_serializer.errors}")

@retry_on_failure('delete_group')
def delete_group(inventory, sender, instance):
    """
    Deletes a group from AWX inventory.
    """
    serializer_class = serializers_dict[sender]
    serializer = serializer_class(instance)
    inventory.delete_group(serializer.data["name"])

def sync_all(job):
    """
    Performs a full synchronization of the AWX inventory.
    """
    inventory = job.object
    logger.info(f"Performing full inventory sync for inventory {inventory.inventory_id}")
    job.start()
    for site in Site.objects.all():
        sync_group(inventory, Site, site)
    for device_role in DeviceRole.objects.all():
        sync_group(inventory, DeviceRole, device_role)
    for device_type in DeviceType.objects.all():
        sync_group(inventory, DeviceType, device_type)
    for tag in Tag.objects.all():
        sync_group(inventory, Tag, tag)

    for device in Device.objects.all():
        if device.primary_ip4 and device.primary_ip4.dns_name:
            sync_host(inventory, Device, device)
    for vm in VirtualMachine.objects.all():
        if vm.primary_ip4 and vm.primary_ip4.dns_name:
            sync_host(inventory, VirtualMachine, vm)
    job.terminate()
